﻿//------------------------------------------------------------------------------
// <copyright file="MainWindow.xaml.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------

namespace Microsoft.Samples.Kinect.DepthBasics
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Windows;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;
    using Microsoft.Kinect;
    using System.Linq; 
    
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Active Kinect sensor
        /// </summary>
        private KinectSensor sensor;

        /// <summary>
        /// Bitmap that will hold color information
        /// </summary>
        private WriteableBitmap colorBitmap;

        /// <summary>
        /// Bitmap that will hold color information
        /// </summary>
        private WriteableBitmap colorBitmap2;

        /// <summary>
        /// Intermediate storage for the depth data received from the camera
        /// </summary>
        private DepthImagePixel[] depthPixels;

        /// <summary>
        /// Intermediate storage for the depth data converted to color
        /// </summary>
        private byte[] colorPixels;

        /// <summary>
        /// Global variables for the enable sound event handler
        /// Default to disabled and no sound alerts
        /// </summary>
        int Dangerlvl = 1;
        bool LightChange = false;
        bool NoiseOption = false;
        bool NoiseFlip = false;

        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Execute startup tasks
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void WindowLoaded(object sender, RoutedEventArgs e)
        {
            // Look through all sensors and start the first connected one.
            // This requires that a Kinect is connected at the time of app startup.
            // To make your app robust against plug/unplug, 
            // it is recommended to use KinectSensorChooser provided in Microsoft.Kinect.Toolkit (See components in Toolkit Browser).
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.sensor = potentialSensor;
                    break;
                }
            }

            if (null != this.sensor)
            {
                // Turn on the color stream to receive color frames
                this.sensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);

                // Turn on the depth stream to receive depth frames
                this.sensor.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30);
                
                // Allocate space to put the depth pixels we'll receive
                this.depthPixels = new DepthImagePixel[this.sensor.DepthStream.FramePixelDataLength];

                // Allocate space to put the color pixels we'll create
                this.colorPixels = new byte[this.sensor.DepthStream.FramePixelDataLength * sizeof(int)];

                // This is the depth bitmap we'll display on-screen
                this.colorBitmap = new WriteableBitmap(this.sensor.DepthStream.FrameWidth, this.sensor.DepthStream.FrameHeight, 96.0, 96.0, PixelFormats.Bgr32, null);

                // This is the color bitmap we'll display on-screen
                this.colorBitmap2 = new WriteableBitmap(this.sensor.DepthStream.FrameWidth, this.sensor.DepthStream.FrameHeight, 96.0, 96.0, PixelFormats.Bgr32, null);

                // Set the image we display to point to the bitmap where we'll put the depth image data
                this.Image.Source = this.colorBitmap;

                // Set the image we display to point to the bitmap where we'll put the color image data
                this.Image2.Source = this.colorBitmap2;

                // Add an event handler to be called whenever there is new depth frame data
                this.sensor.DepthFrameReady += this.SensorDepthFrameReady;

                // Add an event handler to be called whenever there is new color frame data
                this.sensor.ColorFrameReady += this.SensorColorFrameReady;

                // Start the sensor!
                try
                {
                    this.sensor.Start();
                }
                catch (IOException)
                {
                    this.sensor = null;
                }
            }

            if (null == this.sensor)
            {
                this.statusBarText.Text = Properties.Resources.NoKinectReady;
            }
        }

        /// Execute shutdown tasks
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void WindowClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (null != this.sensor)
            {
                this.sensor.Stop();
            }
        }

        /// Event handler for Kinect sensor's ColorFrameReady event
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SensorColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame != null)
                {
                    // Copy the pixel data from the image to a temporary array
                    colorFrame.CopyPixelDataTo(this.colorPixels);
                    
                    // Write the pixel data into our bitmap
                    this.colorBitmap2.WritePixels(
                        new Int32Rect(0, 0, this.colorBitmap.PixelWidth, this.colorBitmap.PixelHeight),
                        this.colorPixels,
                        this.colorBitmap.PixelWidth * sizeof(int),
                        0);
                           
                }
            }
        }


        /// <summary>
        /// Event handler for Kinect sensor's DepthFrameReady event
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void SensorDepthFrameReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            using (DepthImageFrame depthFrame = e.OpenDepthImageFrame())
            {
                if (depthFrame != null)
                {
                    // Copy the pixel data from the image to a temporary array
                    depthFrame.CopyDepthImagePixelDataTo(this.depthPixels);
                    
                    // Get the min and max reliable depth for the current frame
                    int minDepth = depthFrame.MinDepth;
                    int maxDepth = depthFrame.MaxDepth;
                    int ALARGENUMBER = 10000;                        
                    // Convert the depth to RGB
                    int colorPixelIndex = 0;
                    //int[] lenarray = new int[62]; //depthPixels.Length/5000 = 61, ignore 0th index
                    int[] lenarray = new int[123]; //depthPixels.Length/2500 = 123, ignore 0th index
                    int counter = 0;
                    lenarray[0] = ALARGENUMBER; //ignore keeping the first pixel
                    for (int i = 0; i < this.depthPixels.Length; i++)
                    {
                        // Get the depth for this pixel
                        int depth = depthPixels[i].Depth;

                        //If this value is change, must change size of lenarray
                        int pixelOffset = 2500;
                        // To convert to a byte, we're discarding the most-significant
                        // rather than least-significant bits.
                        // We're preserving detail, although the intensity will "wrap."
                        // Values outside the reliable depth range are mapped to 0 (black).

                        // Note: Using conditionals in this loop could degrade performance.
                        // Consider using a lookup table instead when writing production code.
                        // See the KinectDepthViewer class used by the KinectExplorer sample
                        // for a lookup table example.
                      
                          byte intensity = (byte)(depth >= minDepth && depth <= maxDepth ? depth : 0);
                          if (i % pixelOffset == 0)
                          {
                              intensity = 255;
                              if (depth == 0)//putting a 0 in the array would skew the min() used later
                              {
                                  lenarray[counter] = ALARGENUMBER;//safeguard against unrecognized values
                              }
                              else
                              {
                                  lenarray[counter] = depth;//else put the real depth in hotspot array
                              }
                              counter++;
                          }

                        // Write out blue byte
                        this.colorPixels[colorPixelIndex++] = intensity;

                        // Write out green byte
                        this.colorPixels[colorPixelIndex++] = intensity;

                        // Write out red byte                        
                        this.colorPixels[colorPixelIndex++] = intensity;             
                                                
                        // We're outputting BGR, the last byte in the 32 bits is unused so skip it
                        // If we were outputting BGRA, we would write alpha here.
                        ++colorPixelIndex;
                    }
                    
                    // Write the pixel data into our bitmap
                    this.colorBitmap.WritePixels(
                        new Int32Rect(0, 0, this.colorBitmap.PixelWidth, this.colorBitmap.PixelHeight),
                        this.colorPixels,
                        this.colorBitmap.PixelWidth * sizeof(int),
                        0);

                    //Raw data collection
                    int ClosestDistanceMM = lenarray.Min();
                    double ClosestDistanceInch = (ClosestDistanceMM / 25.4);
                    int BASEyellowzoneVal = 60;
                    int BASEredzoneVal = 36;
                    double AccSliderScalar = this.AccSlider.Value;

                    //Adjusted data values
                    double yellowzoneVal = BASEyellowzoneVal *AccSliderScalar;
                    double redzoneVal = BASEredzoneVal *AccSliderScalar;
                  
                    //Textbox update per frame iteration
                    ////Mask for the string function provided by: 
                    ////stackoverflow.com/questions/164926/c-sharp-how-do-i-round-a-decimal-value-to-2-decimal-places-for-output-on-a-pa
                    this.DistanceTB.Text = ClosestDistanceMM.ToString();
                    this.InchesTB.Text = ClosestDistanceInch.ToString("#.##");
                    this.RedZoneTextbox.Text = redzoneVal.ToString("#.##");
                    this.YellowZoneTextbox.Text = yellowzoneVal.ToString("#.##");
                    //Sound variables
                    string localDir = Environment.CurrentDirectory;
                    string YellowSoundPath = Path.Combine(Environment.CurrentDirectory, "YellowAlarm.wav");
                    string RedSoundPath = Path.Combine(localDir, "RedAlarm.wav");
                    System.Media.SoundPlayer Yellowplayer = new System.Media.SoundPlayer(YellowSoundPath);
                    System.Media.SoundPlayer Redplayer = new System.Media.SoundPlayer(RedSoundPath);
                   

                    this.DangerBorder.BorderBrush = Brushes.Green;
                    //Safe (green) until proven unsafe in the following conditionals
                    //redundancy in the following conditionals is to enable responsiveness from the noise alerting event handler
                    if (ClosestDistanceInch < redzoneVal)
                    {            
                        //check for state change
                        if(Dangerlvl != 3)
                        {
                            Dangerlvl = 3;
                            LightChange = true;
                        }                 
                        else
                        {
                            LightChange = false; 
                        }
                        this.DangerBorder.BorderBrush = Brushes.Red;
                    }   
                    else
                    {
                        if (ClosestDistanceInch < yellowzoneVal)
                        {
                            //check for state change
                            if(Dangerlvl != 2)
                            {
                                Dangerlvl = 2;
                                LightChange = true;
                            }  
                            else 
                            {
                                LightChange = false;
                            }                       
                            this.DangerBorder.BorderBrush = Brushes.Yellow;                     
                        }
                        else
                        {
                                //check for state change
                                if(Dangerlvl != 1)
                                {
                                    Dangerlvl = 1;
                                    LightChange = true;
                                }  
                                else 
                                {
                                    LightChange = false;
                                }
                            this.DangerBorder.BorderBrush = Brushes.Green;
                        }
                    }//end else

                   if( NoiseOption == true && LightChange == true)
                   {
                       if(Dangerlvl == 1)
                       {
                           Redplayer.Stop();
                           Yellowplayer.Stop();
                       }
                       if (Dangerlvl == 2)
                       {
                           Redplayer.Stop();
                           Yellowplayer.PlayLooping();
                       }
                       if (Dangerlvl == 3)
                       {
                           Yellowplayer.Stop();
                           Redplayer.PlayLooping();
                       }
                   }


                   if (NoiseFlip == true)
                   {
                       if (NoiseOption == false)
                       {
                           Yellowplayer.Stop();
                           Redplayer.Stop();
                       }
                       else
                       {
                           if (Dangerlvl == 1)
                           {
                               Redplayer.Stop();
                               Yellowplayer.Stop();
                           }
                           if (Dangerlvl == 2)
                           {
                               Redplayer.Stop();
                               Yellowplayer.PlayLooping();
                           }
                           if (Dangerlvl == 3)
                           {
                               Yellowplayer.Stop();
                               Redplayer.PlayLooping();
                           }
                       }
                       NoiseFlip = false;
                   }
                                               
                }
            }//end using DepthImageFrame depthFrame = e.OpenDepthImageFrame()
        }


        /// <summary>
        /// Handles the user clicking on the screenshot button
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void ButtonScreenshotClick(object sender, RoutedEventArgs e)
        {
            if (null == this.sensor)
            {
                this.statusBarText.Text = Properties.Resources.ConnectDeviceFirst;
                return;
            }

            // create a png bitmap encoder which knows how to save a .png file
            BitmapEncoder encoder = new PngBitmapEncoder();

            // create frame from the writable bitmap and add to encoder
            encoder.Frames.Add(BitmapFrame.Create(this.colorBitmap2));

            string time = System.DateTime.Now.ToString("hh'-'mm'-'ss", CultureInfo.CurrentUICulture.DateTimeFormat);
            
            //string myPhotos = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            string myPhotos = Environment.CurrentDirectory;//save relative  to project folder

            string path = Path.Combine(myPhotos, "KinectSnapsh3ot-" + time + ".png");

            // write the new file to disk
            try
            {
                using (FileStream fs = new FileStream(path, FileMode.Create))
                {
                    encoder.Save(fs);
                }

                this.statusBarText.Text = string.Format(CultureInfo.InvariantCulture, "{0} {1}", Properties.Resources.ScreenshotWriteSuccess, path);
            }
            catch (IOException)
            {
                this.statusBarText.Text = string.Format(CultureInfo.InvariantCulture, "{0} {1}", Properties.Resources.ScreenshotWriteFailed, path);
            }
        }
        
        /// <summary>
        /// Handles the checking or unchecking of the near mode combo box
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void CheckBoxNearModeChanged(object sender, RoutedEventArgs e)
        {
            if (this.sensor != null)
            {
                // will not function on non-Kinect for Windows devices
                try
                {
                    if (this.checkBoxNearMode.IsChecked.GetValueOrDefault())
                    {
                        this.sensor.DepthStream.Range = DepthRange.Near;
                    }
                    else
                    {
                        this.sensor.DepthStream.Range = DepthRange.Default;
                    }                
                }
                catch (InvalidOperationException)
                {
                }
            }
        }

        /// <summary>
        /// Handles the checking or unchecking of the noise alerting events. Checked = enabled
        /// </summary>
        /// <param name="sender">object sending the event</param>
        /// <param name="e">event arguments</param>
        private void CheckBoxNoiseChanged(object sender, RoutedEventArgs e)
        {
            if (this.sensor != null)
            {
                // will not function on non-Kinect for Windows devices
                try
                {
                    if (this.checkBoxNoise.IsChecked.GetValueOrDefault())
                    {
                        NoiseOption = true;
                    }
                    else
                    {
                        NoiseOption = false;
                    }
                    NoiseFlip = true;
                }
                catch (InvalidOperationException)
                {
                }
            }
        }
       
    }
}